/*
 *  TOPPERS/C++ Binding
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      C++ Binding
 *
 *  Copyright (C) 2004 by TAKAGI Nobuhisa
 *
 *  嵭Ԥϡʲ (1)(4) ξ狼Free Software Foundation
 *  ˤäƸɽƤ GNU General Public License  Version 2 ˵
 *  ҤƤ˸¤ꡤܥեȥܥեȥ
 *  ѤΤޤࡥʲƱˤѡʣѡۡʰʲ
 *  ѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơŬѲǽ
 *  ޤơʤݾڤԤʤޤܥեȥѤˤľ
 *  ŪޤϴŪʤ»˴ؤƤ⡤Ǥʤ
 *
 *  @(#) $Id: kernel_test_trace.c,v 1.3 2004/01/26 00:53:45 takagi-n Exp $
 */
#include <kernel.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

const char *syscall_name;
VP_INT test_params[6];
int arity;
BOOL _kernel_iniflg;

static BOOL context;
static BOOL cpu_lock;
static BOOL disable_dispatch;
static BOOL disable_task_exception;
static ID current_task = 1;
static long long block[256];

typedef int _toppers_cxxrt_sync_t;
typedef volatile char _toppers_cxxrt_once_t;

#define	_CXXRT_SEM	7

/*
 *  For Syncronization
 */
int _toppers_cxxrt_lock(_toppers_cxxrt_sync_t *_sync)
{
	if (_kernel_iniflg)
		dis_dsp();
	return 0;
}

int _toppers_cxxrt_trylock(_toppers_cxxrt_sync_t *_sync)
{
	return _toppers_cxxrt_lock(_sync);
}

int _toppers_cxxrt_unlock(_toppers_cxxrt_sync_t *_sync)
{
	if (_kernel_iniflg)
		ena_dsp();
	return 0;
}

static inline int _get_tid()
{
	return current_task;
}

int _toppers_cxxrt_get_tskid(void)
{
	return _kernel_iniflg ? _get_tid() : 0;
}

/*
 *  For recursive semaphore lock
 */
static ID _cxxrt_holder;
static UINT _cxxrt_counter;
extern BOOL _kernel_iniflg;

static inline void _toppers_cxxrt_recursive_semaphore_lock(ID semid)
{
	if (_kernel_iniflg && !sns_dsp())
	{
		ID _tskid;
		_tskid = _get_tid();

		if (_cxxrt_holder != _tskid)
		{
			wai_sem(semid);
			_cxxrt_holder = _tskid;
		}
		++_cxxrt_counter;
	}
}

static inline void _toppers_cxxrt_recursive_semaphore_unlock(ID semid)
{
	if (_kernel_iniflg && !sns_dsp())
	{
		if (--_cxxrt_counter == 0)
		{
			_cxxrt_holder = 0;
			sig_sem(semid);
		}
	}
}


/*
 *  For function call once
 */
int _toppers_cxxrt_once(_toppers_cxxrt_once_t *_once, void(*_func)(void))
{
	if (!*_once)
	{
		static _toppers_cxxrt_sync_t _sync;

		_toppers_cxxrt_recursive_semaphore_lock(_CXXRT_SEM);
		if (!*_once)
		{
			(*_func)();
			*_once = 1;
		}
		_toppers_cxxrt_recursive_semaphore_unlock(_CXXRT_SEM);
	}
	return 0;
}


/*
 *  For task-local storage
 */
#ifndef _CXXRT_KEY_MAX
#define _CXXRT_KEY_MAX  2
#endif

struct _toppers_cxxrt_tls
{
	void **_data;
	void (*_dtor)(void*);
};

const ID _kernel_tmax_tskid = 10;

static void **_toppers_cxxrt_data[_CXXRT_KEY_MAX];
static struct _toppers_cxxrt_tls _tls[_CXXRT_KEY_MAX];

int _toppers_cxxrt_key_create(struct _toppers_cxxrt_tls **_key, void(*_dtor)(void*))
{
	struct _toppers_cxxrt_tls *_p;

	for (_p = &_tls[0]; _p < &_tls[_CXXRT_KEY_MAX]; _p++)
	{
		if (_p->_data == NULL)
		{
			_p->_data = _toppers_cxxrt_data[_p - &_tls[0]];
			_p->_dtor = _dtor;
			*_key = _p;
			return 0;
		}
	}
	return -1;
}

int _toppers_cxxrt_key_delete(struct _toppers_cxxrt_tls *_key)
{
  int _i;

  for (_i = 0; _i <= _kernel_tmax_tskid; _i++)
  {
    if (_key->_dtor != 0)
      (*_key->_dtor)(_key->_data[_i]);
  }
  _key->_data = NULL;
  _key->_dtor = 0;
  return 0;
}

void _toppers_cxxrt_reset_specific(void)
{
	ID _tskid = 0;
	struct _toppers_cxxrt_tls *_p;

	if (_kernel_iniflg)
		_tskid = _get_tid();

	for (_p = &_tls[0]; _p < &_tls[_CXXRT_KEY_MAX]; _p++)
	{
		if (_p->_data != NULL)
		{
			void **_data = &_p->_data[_tskid];
			if (*_data != NULL)
			{
				if (_p->_dtor != 0)
					(*_p->_dtor)(*_data);
				*_data = NULL;
			}
		}
	}
}

void init_cxxrt(void)
{
	int _i, _tmax = _kernel_tmax_tskid + 1;
	void **_p = (void**)malloc(sizeof(void*) * _CXXRT_KEY_MAX * _tmax);

	for (_i = 0; _i < _CXXRT_KEY_MAX * _tmax; _i++)
		_p[_i] = NULL;
	for (_i = 0; _i < _CXXRT_KEY_MAX; _i++)
		_toppers_cxxrt_data[_i] = _p + _i * _tmax;
	_kernel_iniflg = TRUE;
}

static void trace(void)
{
	int i;
	printf("%s(", syscall_name);
	for (i = 0; i < arity; i++)
	{
		if (i != 0)
		{
			printf(", ");
		}
		printf("%p", test_params[i]);
	}
	printf(");\n");
}

void set_test_task(ID tskid)
{
	current_task = tskid;
}

void set_test_context(BOOL ctx)
{
	context = !!ctx;
}

ER	cre_tsk(ID tskid, const T_CTSK *pk_ctsk)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	test_params[1] = (VP_INT)pk_ctsk;
	arity = 2;
	trace();
	return E_OK;
}

ER_ID	acre_tsk(const T_CTSK *pk_ctsk)
{
	static ID id = 1;
	syscall_name = __func__;
	test_params[0] = (VP_INT)pk_ctsk;
	arity = 1;
	trace();
	return id++;
}

ER	del_tsk(ID tskid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	arity = 1;
	trace();
	return E_OK;
}

ER	act_tsk(ID tskid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	arity = 1;
	trace();
	return E_OK;
}

ER	iact_tsk(ID tskid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	arity = 1;
	trace();
	return E_OK;
}

ER_UINT	can_act(ID tskid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	arity = 1;
	trace();
	return 1;
}

ER	sta_tsk(ID tskid, VP_INT stacd)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	test_params[1] = (VP_INT)stacd;
	arity = 2;
	trace();
	return E_OK;
}

void	ext_tsk(void)
{
	syscall_name = __func__;
	arity = 0;
	trace();
}

void	exd_tsk(void)
{
	syscall_name = __func__;
	arity = 0;
	trace();
}

ER	ter_tsk(ID tskid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	arity = 1;
	trace();
	return E_OK;
}

ER	chg_pri(ID tskid, PRI tskpri)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	test_params[1] = (VP_INT)tskpri;
	arity = 2;
	trace();
	return E_OK;
}

ER	get_pri(ID tskid, PRI *p_tskpri)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	test_params[1] = (VP_INT)p_tskpri;
	arity = 2;
	trace();
	if (p_tskpri != NULL)
		*p_tskpri = 5;
	return E_OK;
}

ER	ref_tsk(ID tskid, T_RTSK *pk_rtsk)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	test_params[1] = (VP_INT)pk_rtsk;
	arity = 2;
	trace();
	return E_OK;
}

ER	ref_tst(ID tskid, T_RTST *pk_rtst)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	test_params[1] = (VP_INT)pk_rtst;
	arity = 2;
	trace();
	return E_OK;
}

ER	slp_tsk(void)
{
	syscall_name = __func__;
	arity = 0;
	trace();
	return E_OK;
}

ER	tslp_tsk(TMO tmout)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tmout;
	arity = 1;
	trace();
	return E_OK;
}

ER	wup_tsk(ID tskid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	arity = 1;
	trace();
	return E_OK;
}

ER	iwup_tsk(ID tskid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	arity = 1;
	trace();
	return E_OK;
}

ER_UINT	can_wup(ID tskid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	arity = 1;
	trace();
	return E_OK;
}

ER	rel_wai(ID tskid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	arity = 1;
	trace();
	return E_OK;
}

ER	irel_wai(ID tskid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	arity = 1;
	trace();
	return E_OK;
}

ER	sus_tsk(ID tskid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	arity = 1;
	trace();
	return E_OK;
}

ER	rsm_tsk(ID tskid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	arity = 1;
	trace();
	return E_OK;
}

ER	frsm_tsk(ID tskid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	arity = 1;
	trace();
	return E_OK;
}

ER	dly_tsk(RELTIM dlytim)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)dlytim;
	arity = 1;
	trace();
	return E_OK;
}

ER	def_tex(ID tskid, const T_DTEX *pk_dtex)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	test_params[1] = (VP_INT)pk_dtex;
	arity = 2;
	trace();
	return E_OK;
}

ER	ras_tex(ID tskid, TEXPTN rasptn)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	test_params[1] = (VP_INT)rasptn;
	arity = 2;
	trace();
	return E_OK;
}

ER	iras_tex(ID tskid, TEXPTN rasptn)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	test_params[1] = (VP_INT)rasptn;
	arity = 2;
	trace();
	return E_OK;
}

ER	dis_tex(void)
{
	syscall_name = __func__;
	arity = 0;
	trace();
	disable_task_exception = TRUE;
	return E_OK;
}

ER	ena_tex(void)
{
	syscall_name = __func__;
	arity = 0;
	trace();
	disable_task_exception = FALSE;
	return E_OK;
}

BOOL	sns_tex(void)
{
	syscall_name = __func__;
	arity = 0;
	trace();
	return disable_task_exception;
}

ER	ref_tex(ID tskid, T_RTEX *pk_rtex)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	test_params[1] = (VP_INT)pk_rtex;
	arity = 2;
	trace();
	return E_OK;
}

ER	cre_sem(ID semid, const T_CSEM *pk_csem)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)semid;
	test_params[1] = (VP_INT)pk_csem;
	arity = 2;
	trace();
	return E_OK;
}

ER_ID	acre_sem(const T_CSEM *pk_csem)
{
	static ID id = 1;
	syscall_name = __func__;
	test_params[0] = (VP_INT)pk_csem;
	arity = 1;
	trace();
	return id++;
}

ER	del_sem(ID semid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)semid;
	arity = 1;
	trace();
	return E_OK;
}

ER	sig_sem(ID semid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)semid;
	arity = 1;
	trace();
	return E_OK;
}

ER	isig_sem(ID semid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)semid;
	arity = 1;
	trace();
	return E_OK;
}

ER	wai_sem(ID semid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)semid;
	arity = 1;
	trace();
	return E_OK;
}

ER	pol_sem(ID semid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)semid;
	arity = 1;
	trace();
	return E_OK;
}

ER	twai_sem(ID semid, TMO tmout)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)semid;
	test_params[1] = (VP_INT)tmout;
	arity = 2;
	trace();
	return E_OK;
}

ER	ref_sem(ID semid, T_RSEM *pk_rsem)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)semid;
	test_params[1] = (VP_INT)pk_rsem;
	arity = 2;
	trace();
	return E_OK;
}

ER	cre_flg(ID flgid, const T_CFLG *pk_cflg)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)flgid;
	test_params[1] = (VP_INT)pk_cflg;
	arity = 2;
	trace();
	return E_OK;
}

ER_ID	acre_flg(const T_CFLG *pk_cflg)
{
	static ID id = 1;
	syscall_name = __func__;
	test_params[0] = (VP_INT)pk_cflg;
	arity = 1;
	trace();
	return id++;
}

ER	del_flg(ID flgid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)flgid;
	arity = 1;
	trace();
	return E_OK;
}

ER	set_flg(ID flgid, FLGPTN setptn)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)flgid;
	test_params[1] = (VP_INT)setptn;
	arity = 2;
	trace();
	return E_OK;
}

ER	iset_flg(ID flgid, FLGPTN setptn)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)flgid;
	test_params[1] = (VP_INT)setptn;
	arity = 2;
	trace();
	return E_OK;
}

ER	clr_flg(ID flgid, FLGPTN clrptn)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)flgid;
	test_params[1] = (VP_INT)clrptn;
	arity = 2;
	trace();
	return E_OK;
}

ER	wai_flg(ID flgid, FLGPTN waiptn,
		MODE wfmode, FLGPTN *p_flgptn)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)flgid;
	test_params[1] = (VP_INT)waiptn;
	test_params[2] = (VP_INT)wfmode;
	test_params[3] = (VP_INT)p_flgptn;
	arity = 4;
	trace();
	return E_OK;
}

ER	pol_flg(ID flgid, FLGPTN waiptn,
		MODE wfmode, FLGPTN *p_flgptn)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)flgid;
	test_params[1] = (VP_INT)waiptn;
	test_params[2] = (VP_INT)wfmode;
	test_params[3] = (VP_INT)p_flgptn;
	arity = 4;
	trace();
	return E_OK;
}

ER	twai_flg(ID flgid, FLGPTN waiptn,
		MODE wfmode, FLGPTN *p_flgptn, TMO tmout)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)flgid;
	test_params[1] = (VP_INT)waiptn;
	test_params[2] = (VP_INT)wfmode;
	test_params[3] = (VP_INT)p_flgptn;
	test_params[4] = (VP_INT)tmout;
	arity = 5;
	trace();
	return E_OK;
}

ER	ref_flg(ID flgid, T_RFLG *pk_rflg)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)flgid;
	test_params[1] = (VP_INT)pk_rflg;
	arity = 2;
	trace();
	return E_OK;
}

ER	cre_dtq(ID dtqid, const T_CDTQ *pk_cdtq)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)dtqid;
	test_params[1] = (VP_INT)pk_cdtq;
	arity = 2;
	trace();
	return E_OK;
}

ER_ID	acre_dtq(const T_CDTQ *pk_cdtq)
{
	static ID id = 1;
	syscall_name = __func__;
	test_params[0] = (VP_INT)pk_cdtq;
	arity = 1;
	trace();
	return id++;
}

ER	del_dtq(ID dtqid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)dtqid;
	arity = 1;
	trace();
	return E_OK;
}

ER	snd_dtq(ID dtqid, VP_INT data)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)dtqid;
	test_params[1] = (VP_INT)data;
	arity = 2;
	trace();
	return E_OK;
}

ER	psnd_dtq(ID dtqid, VP_INT data)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)dtqid;
	test_params[1] = (VP_INT)data;
	arity = 2;
	trace();
	return E_OK;
}

ER	ipsnd_dtq(ID dtqid, VP_INT data)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)dtqid;
	test_params[1] = (VP_INT)data;
	arity = 2;
	trace();
	return E_OK;
}

ER	tsnd_dtq(ID dtqid, VP_INT data, TMO tmout)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)dtqid;
	test_params[1] = (VP_INT)data;
	test_params[2] = (VP_INT)tmout;
	arity = 3;
	trace();
	return E_OK;
}

ER	fsnd_dtq(ID dtqid, VP_INT data)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)dtqid;
	test_params[1] = (VP_INT)data;
	arity = 2;
	trace();
	return E_OK;
}

ER	ifsnd_dtq(ID dtqid, VP_INT data)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)dtqid;
	test_params[1] = (VP_INT)data;
	arity = 2;
	trace();
	return E_OK;
}

ER	rcv_dtq(ID dtqid, VP_INT *p_data)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)dtqid;
	test_params[1] = (VP_INT)p_data;
	arity = 2;
	trace();
	return E_OK;
}

ER	prcv_dtq(ID dtqid, VP_INT *p_data)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)dtqid;
	test_params[1] = (VP_INT)p_data;
	arity = 2;
	trace();
	return E_OK;
}

ER	trcv_dtq(ID dtqid, VP_INT *p_data, TMO tmout)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)dtqid;
	test_params[1] = (VP_INT)p_data;
	test_params[2] = (VP_INT)tmout;
	arity = 3;
	trace();
	return E_OK;
}

ER	ref_dtq(ID dtqid, T_RDTQ *pk_rdtq)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)dtqid;
	test_params[1] = (VP_INT)pk_rdtq;
	arity = 2;
	trace();
	return E_OK;
}


ER	cre_mbx(ID mbxid, const T_CMBX *pk_cmbx)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mbxid;
	test_params[1] = (VP_INT)pk_cmbx;
	arity = 2;
	trace();
	return E_OK;
}

ER_ID	acre_mbx(const T_CMBX *pk_cmbx)
{
	static ID id = 1;
	syscall_name = __func__;
	test_params[0] = (VP_INT)pk_cmbx;
	arity = 1;
	trace();
	return id++;
}

ER	del_mbx(ID mbxid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mbxid;
	arity = 1;
	trace();
	return E_OK;
}

ER	snd_mbx(ID mbxid, T_MSG *pk_msg)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mbxid;
	test_params[1] = (VP_INT)pk_msg;
	arity = 2;
	trace();
	return E_OK;
}

ER	rcv_mbx(ID mbxid, T_MSG **ppk_msg)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mbxid;
	test_params[1] = (VP_INT)ppk_msg;
	arity = 2;
	trace();
	return E_OK;
}

ER	prcv_mbx(ID mbxid, T_MSG **ppk_msg)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mbxid;
	test_params[1] = (VP_INT)ppk_msg;
	arity = 2;
	trace();
	return E_OK;
}

ER	trcv_mbx(ID mbxid, T_MSG **ppk_msg, TMO tmout)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mbxid;
	test_params[1] = (VP_INT)ppk_msg;
	test_params[2] = (VP_INT)tmout;
	arity = 3;
	trace();
	return E_OK;
}

ER	ref_mbx(ID mbxid, T_RMBX *pk_rmbx)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mbxid;
	test_params[1] = (VP_INT)pk_rmbx;
	arity = 2;
	trace();
	return E_OK;
}

ER	cre_mtx(ID mtxid, const T_CMTX *pk_cmtx)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mtxid;
	test_params[1] = (VP_INT)pk_cmtx;
	arity = 2;
	trace();
	return E_OK;
}

ER_ID	acre_mtx(const T_CMTX *pk_cmtx)
{
	static ID id = 1;
	syscall_name = __func__;
	test_params[0] = (VP_INT)pk_cmtx;
	arity = 1;
	trace();
	return id++;
}

ER	del_mtx(ID mtxid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mtxid;
	arity = 1;
	trace();
	return E_OK;
}

ER	loc_mtx(ID mtxid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mtxid;
	arity = 1;
	trace();
	return E_OK;
}

ER	ploc_mtx(ID mtxid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mtxid;
	arity = 1;
	trace();
	return E_OK;
}

ER	tloc_mtx(ID mtxid, TMO tmout)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mtxid;
	test_params[1] = (VP_INT)tmout;
	arity = 2;
	trace();
	return E_OK;
}

ER	unl_mtx(ID mtxid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mtxid;
	arity = 1;
	trace();
	return E_OK;
}

ER	ref_mtx(ID mtxid, T_RMTX *pk_cmtx)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mtxid;
	test_params[1] = (VP_INT)pk_cmtx;
	arity = 2;
	trace();
	return E_OK;
}

ER	cre_mbf(ID mbfid, const T_CMBF *pk_cmbf)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mbfid;
	test_params[1] = (VP_INT)pk_cmbf;
	arity = 2;
	trace();
	return E_OK;
}

ER_ID	acre_mbf(const T_CMBF *pk_cmbf)
{
	static ID id = 1;
	syscall_name = __func__;
	test_params[0] = (VP_INT)pk_cmbf;
	arity = 1;
	trace();
	return id++;
}

ER	del_mbf(ID mbfid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mbfid;
	arity = 1;
	trace();
	return E_OK;
}

ER	snd_mbf(ID mbfid, VP msg, UINT msgsz)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mbfid;
	test_params[1] = (VP_INT)msg;
	test_params[2] = (VP_INT)msgsz;
	arity = 3;
	trace();
	return E_OK;
}

ER	psnd_mbf(ID mbfid, VP msg, UINT msgsz)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mbfid;
	test_params[1] = (VP_INT)msg;
	test_params[2] = (VP_INT)msgsz;
	arity = 3;
	trace();
	return E_OK;
}

ER	tsnd_mbf(ID mbfid, VP msg, UINT msgsz, TMO tmout)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mbfid;
	test_params[1] = (VP_INT)msg;
	test_params[2] = (VP_INT)msgsz;
	test_params[3] = (VP_INT)tmout;
	arity = 4;
	trace();
	return E_OK;
}

ER_UINT	rcv_mbf(ID mbfid, VP msg)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mbfid;
	test_params[1] = (VP_INT)msg;
	arity = 2;
	trace();
	return E_OK;
}

ER_UINT	prcv_mbf(ID mbfid, VP msg)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mbfid;
	test_params[1] = (VP_INT)msg;
	arity = 2;
	trace();
	return E_OK;
}

ER_UINT	trcv_mbf(ID mbfid, VP msg, TMO tmout)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mbfid;
	test_params[1] = (VP_INT)msg;
	test_params[2] = (VP_INT)tmout;
	arity = 3;
	trace();
	return E_OK;
}

ER	ref_mbf(ID mbfid, T_RMBF *pk_cmbf)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mbfid;
	test_params[1] = (VP_INT)pk_cmbf;
	arity = 2;
	trace();
	return E_OK;
}

ER	cre_por(ID porid, const T_CPOR *pk_cpor)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)porid;
	test_params[1] = (VP_INT)pk_cpor;
	arity = 2;
	trace();
	return E_OK;
}

ER_ID	acre_por(const T_CPOR *pk_cpor)
{
	static ID id = 1;
	syscall_name = __func__;
	test_params[0] = (VP_INT)pk_cpor;
	arity = 1;
	trace();
	return id++;
}

ER	del_por(ID porid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)porid;
	arity = 1;
	trace();
	return E_OK;
}

ER_UINT	cal_por(ID porid, RDVPTN calptn, VP msg, UINT cmsgsz)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)porid;
	test_params[1] = (VP_INT)calptn;
	test_params[2] = (VP_INT)msg;
	test_params[3] = (VP_INT)cmsgsz;
	arity = 4;
	trace();
	return E_OK;
}

ER_UINT	tcal_por(ID porid, RDVPTN calptn, VP msg, UINT cmsgsz, TMO tmout)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)porid;
	test_params[1] = (VP_INT)calptn;
	test_params[2] = (VP_INT)msg;
	test_params[3] = (VP_INT)cmsgsz;
	test_params[4] = (VP_INT)tmout;
	arity = 5;
	trace();
	return E_OK;
}

ER_UINT	acp_por(ID porid, RDVPTN acpptn, RDVNO *p_rdvno, VP msg)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)porid;
	test_params[1] = (VP_INT)acpptn;
	test_params[2] = (VP_INT)p_rdvno;
	test_params[3] = (VP_INT)msg;
	arity = 4;
	trace();
	*p_rdvno = 12345;
	return 100;
}

ER_UINT	pacp_por(ID porid, RDVPTN acpptn, RDVNO *p_rdvno, VP msg)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)porid;
	test_params[1] = (VP_INT)acpptn;
	test_params[2] = (VP_INT)p_rdvno;
	test_params[3] = (VP_INT)msg;
	arity = 4;
	trace();
	*p_rdvno = 12345;
	return 100;
}

ER_UINT	tacp_por(ID porid, RDVPTN acpptn, RDVNO *p_rdvno, VP msg, TMO tmout)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)porid;
	test_params[1] = (VP_INT)acpptn;
	test_params[2] = (VP_INT)p_rdvno;
	test_params[3] = (VP_INT)msg;
	test_params[4] = (VP_INT)tmout;
	arity = 5;
	trace();
	*p_rdvno = 12345;
	return 100;
}

ER	fwd_por(ID porid, RDVPTN calptn, RDVNO rdvno, VP msg, UINT cmsgsz)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)porid;
	test_params[1] = (VP_INT)calptn;
	test_params[2] = (VP_INT)rdvno;
	test_params[3] = (VP_INT)msg;
	test_params[4] = (VP_INT)cmsgsz;
	arity = 5;
	trace();
	return E_OK;
}

ER	rpl_rdv(RDVNO rdvno, VP msg, UINT rmsgsz)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)rdvno;
	test_params[1] = (VP_INT)msg;
	test_params[2] = (VP_INT)rmsgsz;
	arity = 3;
	trace();
	return E_OK;
}

ER	ref_por(ID porid, T_RPOR *pk_rpor)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)porid;
	test_params[1] = (VP_INT)pk_rpor;
	arity = 2;
	trace();
	return E_OK;
}

ER	ref_rdv(RDVNO rdvno, T_RRDV *pk_rrdv)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)rdvno;
	test_params[1] = (VP_INT)pk_rrdv;
	arity = 2;
	trace();
	return E_OK;
}

ER	cre_mpf(ID mpfid, const T_CMPF *pk_cmpf)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mpfid;
	test_params[1] = (VP_INT)pk_cmpf;
	arity = 2;
	trace();
	return E_OK;
}

ER_ID	acre_mpf(const T_CMPF *pk_cmpf)
{
	static ID id = 1;
	syscall_name = __func__;
	test_params[0] = (VP_INT)pk_cmpf;
	arity = 1;
	trace();
	return id++;
}

ER	del_mpf(ID mpfid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mpfid;
	arity = 1;
	trace();
	return E_OK;
}

ER	get_mpf(ID mpfid, VP *p_blk)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mpfid;
	test_params[1] = (VP_INT)p_blk;
	arity = 2;
	trace();
	if (p_blk)
		*p_blk = block;
	return E_OK;
}

ER	pget_mpf(ID mpfid, VP *p_blk)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mpfid;
	test_params[1] = (VP_INT)p_blk;
	arity = 2;
	trace();
	if (p_blk)
		*p_blk = block;
	return E_OK;
}

ER	tget_mpf(ID mpfid, VP *p_blk, TMO tmout)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mpfid;
	test_params[1] = (VP_INT)p_blk;
	test_params[2] = (VP_INT)tmout;
	arity = 3;
	trace();
	if (p_blk)
		*p_blk = block;
	return E_OK;
}

ER	rel_mpf(ID mpfid, VP blk)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mpfid;
	test_params[1] = (VP_INT)blk;
	arity = 2;
	trace();
	return E_OK;
}

ER	ref_mpf(ID mpfid, T_RMPF *pk_rmpf)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mpfid;
	test_params[1] = (VP_INT)pk_rmpf;
	arity = 2;
	trace();
	return E_OK;
}

ER	cre_mpl(ID mplid, const T_CMPL *pk_cmpl)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mplid;
	test_params[1] = (VP_INT)pk_cmpl;
	arity = 2;
	trace();
	return E_OK;
}

ER_ID	acre_mpl(const T_CMPL *pk_cmpl)
{
	static ID id = 1;
	syscall_name = __func__;
	test_params[0] = (VP_INT)pk_cmpl;
	arity = 1;
	trace();
	return id++;
}

ER	del_mpl(ID mplid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mplid;
	arity = 1;
	trace();
	return E_OK;
}

ER	get_mpl(ID mplid, UINT blksz, VP *p_blk)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mplid;
	test_params[1] = (VP_INT)blksz;
	test_params[2] = (VP_INT)p_blk;
	arity = 3;
	trace();
	if (p_blk)
		*p_blk = block;
	return E_OK;
}

ER	pget_mpl(ID mplid, UINT blksz, VP *p_blk)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mplid;
	test_params[1] = (VP_INT)blksz;
	test_params[2] = (VP_INT)p_blk;
	arity = 3;
	trace();
	if (p_blk)
		*p_blk = block;
	return E_OK;
}

ER	tget_mpl(ID mplid, UINT blksz, VP *p_blk, TMO tmout)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mplid;
	test_params[1] = (VP_INT)blksz;
	test_params[2] = (VP_INT)p_blk;
	test_params[3] = (VP_INT)tmout;
	arity = 4;
	trace();
	if (p_blk)
		*p_blk = block;
	return E_OK;
}

ER	rel_mpl(ID mplid, VP blk)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mplid;
	test_params[1] = (VP_INT)blk;
	arity = 2;
	trace();
	return E_OK;
}

ER	ref_mpl(ID mplid, T_RMPL *pk_rmpl)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)mplid;
	test_params[1] = (VP_INT)pk_rmpl;
	arity = 2;
	trace();
	return E_OK;
}

ER	set_tim(const SYSTIM *p_systim)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)p_systim;
	arity = 1;
	trace();
	return E_OK;
}

ER	get_tim(SYSTIM *p_systim)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)p_systim;
	arity = 1;
	trace();
	return E_OK;
}

ER	isig_tim(void)
{
	syscall_name = __func__;
	arity = 0;
	trace();
	return E_OK;
}

ER	cre_cyc(ID cycid, const T_CCYC *pk_ccyc)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)cycid;
	test_params[1] = (VP_INT)pk_ccyc;
	arity = 2;
	trace();
	return E_OK;
}

ER_ID	acre_cyc(const T_CCYC *pk_ccyc)
{
	static ID id = 1;
	syscall_name = __func__;
	test_params[0] = (VP_INT)pk_ccyc;
	arity = 1;
	trace();
	return id++;
}

ER	del_cyc(ID cycid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)cycid;
	arity = 1;
	trace();
	return E_OK;
}

ER	sta_cyc(ID cycid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)cycid;
	arity = 1;
	trace();
	return E_OK;
}

ER	stp_cyc(ID cycid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)cycid;
	arity = 1;
	trace();
	return E_OK;
}

ER	ref_cyc(ID cycid, T_RCYC *pk_rcyc)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)cycid;
	test_params[1] = (VP_INT)pk_rcyc;
	arity = 2;
	trace();
	return E_OK;
}

ER	cre_alm(ID almid, const T_CALM *pk_calm)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)almid;
	test_params[1] = (VP_INT)pk_calm;
	arity = 2;
	trace();
	return E_OK;
}

ER_ID	acre_alm(const T_CALM *pk_calm)
{
	static ID id = 1;
	syscall_name = __func__;
	test_params[0] = (VP_INT)pk_calm;
	arity = 1;
	trace();
	return id++;
}

ER	del_alm(ID almid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)almid;
	arity = 1;
	trace();
	return E_OK;
}

ER	sta_alm(ID almid, RELTIM almtim)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)almid;
	test_params[1] = (VP_INT)almtim;
	arity = 2;
	trace();
	return E_OK;
}

ER	stp_alm(ID almid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)almid;
	arity = 1;
	trace();
	return E_OK;
}

ER	ref_alm(ID almid, T_RALM *pk_ralm)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)almid;
	test_params[1] = (VP_INT)pk_ralm;
	arity = 2;
	trace();
	return E_OK;
}

ER	def_ovr(const T_DOVR *pk_dovr)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)pk_dovr;
	arity = 1;
	trace();
	return E_OK;
}

ER	sta_ovr(ID tskid, OVRTIM ovrtim)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	test_params[1] = (VP_INT)ovrtim;
	arity = 2;
	trace();
	return E_OK;
}

ER	stp_ovr(ID tskid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	arity = 1;
	trace();
	return E_OK;
}

ER	ref_ovr(ID tskid, T_ROVR *pk_rovr)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskid;
	test_params[1] = (VP_INT)pk_rovr;
	arity = 2;
	trace();
	return E_OK;
}

ER	rot_rdq(PRI tskpri)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskpri;
	arity = 1;
	trace();
	return E_OK;
}

ER	irot_rdq(PRI tskpri)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)tskpri;
	arity = 1;
	trace();
	return E_OK;
}

ER	get_tid(ID *p_tskid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)p_tskid;
	arity = 1;
	trace();
	if (p_tskid == NULL)
		return E_PAR;
	*p_tskid = current_task;
	return E_OK;
}

ER	iget_tid(ID *p_tskid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)p_tskid;
	arity = 1;
	trace();
	if (p_tskid == NULL)
		return E_PAR;
	*p_tskid = current_task;
	return E_OK;
}

ER	loc_cpu(void)
{
	syscall_name = __func__;
	arity = 0;
	trace();
	cpu_lock = TRUE;
	return E_OK;
}

ER	iloc_cpu(void)
{
	syscall_name = __func__;
	arity = 0;
	trace();
	cpu_lock = TRUE;
	return E_OK;
}

ER	unl_cpu(void)
{
	syscall_name = __func__;
	arity = 0;
	trace();
	cpu_lock = FALSE;
	return E_OK;
}

ER	iunl_cpu(void)
{
	syscall_name = __func__;
	arity = 0;
	trace();
	cpu_lock = FALSE;
	return E_OK;
}

ER	dis_dsp(void)
{
	syscall_name = __func__;
	arity = 0;
	trace();
	disable_dispatch = TRUE;
	return E_OK;
}

ER	ena_dsp(void)
{
	syscall_name = __func__;
	arity = 0;
	trace();
	disable_dispatch = FALSE;
	return E_OK;
}

BOOL	sns_ctx(void)
{
	syscall_name = __func__;
	arity = 0;
	trace();
	return context;
}

BOOL	sns_loc(void)
{
	syscall_name = __func__;
	arity = 0;
	trace();
	return cpu_lock;
}

BOOL	sns_dsp(void)
{
	syscall_name = __func__;
	arity = 0;
	trace();
	return disable_dispatch;
}

BOOL	sns_dpn(void)
{
	syscall_name = __func__;
	arity = 0;
	trace();
	return cpu_lock || disable_dispatch;
}

BOOL	vsns_ini(void)
{
	syscall_name = __func__;
	arity = 0;
	trace();
	return !_kernel_iniflg;
}

ER	ref_sys(T_RSYS *pk_rsys)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)pk_rsys;
	arity = 1;
	trace();
	return E_OK;
}

ER def_inh(INHNO inhno, const T_DINH *pk_dinh)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)inhno;
	test_params[1] = (VP_INT)pk_dinh;
	arity = 2;
	trace();
	return E_OK;
}

ER cre_isr(ID isrid, const T_CISR *pk_cisr)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)isrid;
	test_params[1] = (VP_INT)pk_cisr;
	arity = 2;
	trace();
	return E_OK;
}

ER_ID acre_isr(const T_CISR *pk_cisr)
{
	static ID id = 1;
	syscall_name = __func__;
	test_params[0] = (VP_INT)pk_cisr;
	arity = 1;
	trace();
	return id++;
}

ER del_isr(ID isrid)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)isrid;
	arity = 1;
	trace();
	return E_OK;
}

ER ref_isr(ID isrid, T_RISR *pk_risr)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)isrid;
	test_params[1] = (VP_INT)pk_risr;
	arity = 2;
	trace();
	return E_OK;
}

ER dis_int(INTNO intno)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)intno;
	arity = 1;
	trace();
	return E_OK;
}

ER ena_int(INTNO intno)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)intno;
	arity = 1;
	trace();
	return E_OK;
}

ER chg_ipm(IPM ipm)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)ipm;
	arity = 1;
	trace();
	return E_OK;
}

ER get_ipm(IPM *p_ipm)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)p_ipm;
	arity = 1;
	trace();
	return E_OK;
}

ER def_svc(FN fncd, const T_DSVC *pk_dsvc)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)fncd;
	test_params[1] = (VP_INT)pk_dsvc;
	arity = 2;
	trace();
	return E_OK;
}

ER_UINT cal_svc(FN fncd, VP_INT par1, VP_INT par2, VP_INT par3, VP_INT par4, VP_INT par5)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)fncd;
	test_params[1] = (VP_INT)par1;
	test_params[2] = (VP_INT)par2;
	test_params[3] = (VP_INT)par3;
	test_params[4] = (VP_INT)par4;
	test_params[5] = (VP_INT)par5;
	arity = 6;
	trace();
	return E_OK;
}

ER def_exc(EXCNO excno, const T_DEXC *pk_dexc)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)excno;
	test_params[1] = (VP_INT)pk_dexc;
	arity = 2;
	trace();
	return E_OK;
}

ER ref_cfg(T_RCFG *pk_rcfg)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)pk_rcfg;
	arity = 1;
	trace();
	return E_OK;
}

ER ref_ver(T_RVER *pk_rver)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)pk_rver;
	arity = 1;
	trace();
	return E_OK;
}

BOOL	vxsns_ctx(VP p_excinf)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)p_excinf;
	arity = 1;
	trace();
	return FALSE;
}

BOOL	vxsns_loc(VP p_excinf)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)p_excinf;
	arity = 1;
	trace();
	return FALSE;
}

BOOL	vxsns_dsp(VP p_excinf)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)p_excinf;
	arity = 1;
	trace();
	return FALSE;
}

BOOL	vxsns_dpn(VP p_excinf)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)p_excinf;
	arity = 1;
	trace();
	return FALSE;
}

BOOL	vxsns_tex(VP p_excinf)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)p_excinf;
	arity = 1;
	trace();
	return FALSE;
}

ER	vxget_tim(SYSUTIM* p_sysutim)
{
	syscall_name = __func__;
	test_params[0] = (VP_INT)p_sysutim;
	arity = 1;
	trace();
	return E_OK;
}

ER	chg_ims(IMS ims)
{
	return chg_ipm(ims);
}

ER	get_ims(IMS *p_ims)
{
	return get_ipm(p_ims);
}

ER	vwri_log(UINT prio, SYSLOG *p_log)
{
	static char s[1024];
	if (p_log == NULL)
		return E_PAR;
	sprintf(s, (const char*)p_log->loginfo[0], p_log->loginfo[1], p_log->loginfo[2], p_log->loginfo[3], p_log->loginfo[4], p_log->loginfo[5]);
	puts(s);
	test_params[0] = (VP_INT)s;
	return E_OK;
}

