/*
 *  TOPPERS/C++ Binding
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      C++ Binding
 *
 *  Copyright (C) 2004 by TAKAGI Nobuhisa
 *
 *  嵭Ԥϡʲ (1)(4) ξ狼Free Software Foundation
 *  ˤäƸɽƤ GNU General Public License  Version 2 ˵
 *  ҤƤ˸¤ꡤܥեȥܥեȥ
 *  ѤΤޤࡥʲƱˤѡʣѡۡʰʲ
 *  ѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơŬѲǽ
 *  ޤơʤݾڤԤʤޤܥեȥѤˤľ
 *  ŪޤϴŪʤ»˴ؤƤ⡤Ǥʤ
 *
 *  @(#) $Id: allocator.hpp,v 1.3 2004/01/26 00:53:45 takagi-n Exp $
 */

/**
 *	@file	toppers/allocator.hpp
 *	@brief	ꥢ˴ؤ륯饹
 *	 ΥإåեǤϰʲΥ饹Ƥ롣
 *
 *	@code
 *	class allocator<T, Pool>;
 *	class fixed_sized_allocator<Size, Pool>;
 *	class fixed_sized_new<Host, Pool, Allocator>;
 *	class variable_sized_new<Host, Pool>;
 *	class fixed_simple_pool<N, Size>;
 *	class variable_simple_pool<TotalSize>;
 *	class fixed_new_pool<Size>;
 *	class variable_new_pool;
 *	@endcode
 */
#ifndef	TOPPERS_ALLOCATOR_HPP_
#define	TOPPERS_ALLOCATOR_HPP_

#include <toppers/fmempool.hpp>
#include <toppers/vmempool.hpp>
#include <toppers/sync.hpp>
#include <new>
#include <cstddef>

namespace toppers
{

	template <typename T>
	class alignof
	{
		struct _
		{
			char	a_;
			T		b_;
		};

	public:
		static const std::size_t value = offsetof(_, b_);
	};

	namespace detail
	{

		union max_alignment_helper
		{
			long l;
#ifdef	TOPPERS_LONG_LONG
			TOPPERS_LONG_LONG ll;
#endif
			double d;
			long double ld;
		};

	}

	const std::size_t max_alignment = alignof<detail::max_alignment_helper>::value;

	template <typename T>
	class rounded_sizeof
	{
	public:
		static const std::size_t value = (sizeof(T) + max_alignment - 1) / max_alignment * max_alignment;
	};

	inline void throw_bad_alloc()
	{
		static std::bad_alloc x;
		throw x;
	}

	/**
	 *	@class	allocator allocator.hpp <toppers/allocator.hpp>
	 *	@brief	סѤSTLߴΥ饹
	 *	@ingroup	Allocator
	 *
	 *	 Υƥץ졼ȥ饹ϡSTLѤǤĶˤơƥ
	 *	ȤƻѤ뤿׷Ƥ롣
	 *	ITRONΥסѤơ@e allocator 򥫥ޥݤ˻
	 *	뤳ȤǤ롣
	 */
	template <typename T, class Pool>
	class allocator
	{
		Pool pool_;

	public:
		///	֥ȤΥ
		typedef	std::size_t		size_type;
		///	ƥ졼ʤξϥݥ󥿡˴֤εΥ
		typedef std::ptrdiff_t	different_type;
		///	ݥ󥿷
		typedef T*				pointer;
		///	ݥ󥿷
		typedef const T*		const_pointer;
		///	ȷ
		typedef T&				reference;
		///	ȷ
		typedef const T&		const_reference;
		///	֥Ȥη
		typedef T				value_type;

		///	ۤʤ뷿Υ֥Ȥդǽˤ뤿Υƥץ졼ȥ饹
		template <typename T1>
		struct rebind
		{
		 	typedef	allocator<T1, Pool>	other;
		};

		///	󥹥ȥ饯
		allocator() throw() {}
		///	סΥ󥹥ȥ饯
		explicit allocator(const Pool& pool) throw() : pool_(pool) {}
		///	ԡ󥹥ȥ饯
		allocator(const allocator&) throw() {}
		///	ƥץ졼Ȱ㤤Ѵ󥹥ȥ饯
		template <typename T1>
		allocator(const allocator<T1, Pool>&) throw() {}
		///	ǥȥ饯
		~allocator() throw() {}

		///	ꥪ֥ȤΥɥ쥹֤
		pointer address(reference x) const { return &x; }
		///	ꥪ֥ȤΥɥ쥹֤
		const_pointer address(const_reference x) const { return &x; }

		///	֥åγդ
		T* allocate(size_type n, const void* = 0)
		{
			if (n > 0)
			{
				void* p;
				if (pool_.get(n * sizeof(T), &p) == E_OK)
					return static_cast<T*>(p);
				throw_bad_alloc();
			}
			return 0;
		}
		///	֥åβ
		void deallocate(pointer p, size_type)
		{
			pool_.release(p);
		}

		///	ղǽʺǿ
		size_type max_size() const throw()
		{
			return ~size_type(0) / sizeof(T);
		}

		///	֥å@e T Υ󥹥󥹤Ȥƽ
		void construct(pointer p, const T& value)
		{
			new(p) T(value);
		}
		///	֥å@e T Υ󥹥󥹤Ȥƺ
		void destroy(pointer p)
		{
			p->~T();
		}
	};

	/**
	 *	@class	fixed_sized_allocator allocator.hpp <toppers/allocator.hpp>
	 *	@brief	ꥵΥ뤿Υإѡ饹
	 *	@ingroup	Allocator
	 *
	 *	 @e fixed_sized_allocator ƥץ졼ȥ饹ϡĹΥ֥åդ뤿
	 *	إѡ饹Ǥ롣
	 *	Υإѡ饹ϡȤ@e fixed_sized_new Υƥץ졼Ȱ@e Allocator Τ
	 *	Ѥ롣
	 */
	template <std::size_t Size, class Pool>
	class fixed_sized_allocator
	{
		void (*handler_)();
		Pool pool_;

	public:
		typedef void (*handler_type)();

		explicit fixed_sized_allocator(handler_type handler = 0) : handler_(handler) {}

		///	֥åγդ
		template <typename Sync>
		void* allocate(Sync sync)
		{
			void* p;
			while (pool_.get(Size, &p, sync) != E_OK)
			{
				if (handler_ == 0)
					throw_bad_alloc();
				(*handler_)();
			}
			return p;
		}
		///	֥åγդʱʵԤ
		void* allocate()
		{
			return allocate(forever);
		}
		///	֥åβ
		void deallocate(void* p) throw()
		{
			pool_.release(p);
		}
		///	class_level_allocator Τ set_new_handler ؿ
		handler_type set_new_handler(handler_type handler)
		{
			handler_type prev = handler_;
			handler_ = handler;
			return prev;
		}
	};


	/**
	 *	@class	fixed_sized_new allocator.hpp <toppers/allocator.hpp>
	 *	@brief	סѤ new  delete 黻
	 *	@ingroup	Allocator
	 *
	 *	 @e fixed_sized_new ϡƥץ졼Ȱ@e Pool ǻꤷסѤ new  delete 
	 *	黻ҤƤ롣\n
	 *	 @e fixed_sized_new  public Ѿ뤳ȤˤꡢΥ饹 new  delete 黻Ҥ򥰥Х
	 *	new  delete 黻Ҥ֤뤳ȤǤ롣Ѥ new  delete 黻ҤϥǥեȤ
	 *	ΤѤ롣\n
	 *	 ƥץ졼Ȱ@e Host ˤϡ@e variable_sized_new 륯饹ΤΤꤹ뤳ȡ
	 *	ޤƥץ졼ȥƥץ졼Ȱ@e Allocator ϡ@e Host Υ֥ȥꤷƸĹ֥å
	 *	դ뤿Υإѡƥץ졼ȤǤ뤬ۤȤɤξϥǥեȤΤޤޤǤޤʤ
	 */
	template <class Host, class Pool, template <std::size_t, class> class Allocator = fixed_sized_allocator>
	class fixed_sized_new
	{
	protected:
		///	ǥȥ饯
		/**
		 *	fixed_sized_newΥݥ󥿤Ѥ饹delete뤳Ȥʤ褦
		 *	protected°Ȥ롣
		 */
		~fixed_sized_new() {}

	public:
		typedef void (*handler_type)();

		///	new黻
		static void* operator new(std::size_t) throw(std::bad_alloc)
		{
			return allocator<rounded_sizeof<Host>::value>::impl_.allocate();
		}
		///	delete黻
		static void operator delete(void* p) throw()
		{
			allocator<rounded_sizeof<Host>::value>::impl_.deallocate(p);
		}

		///	Ʊˡդnew黻
		/**
		 *	@param	sync	ƱˡforeverpollingӥॢȻ֤Ǥ롣
		 */
		template <typename Sync>
		static void* operator new(std::size_t, Sync sync) throw(std::bad_alloc)
		{
			return allocator<rounded_sizeof<Host>::value>::impl_.allocate(sync);
		}
		///	Ʊˡդnew黻Ҥбdelete黻
		template <typename Sync>
		static void operator delete(void* p, Sync) throw()
		{
			allocator<rounded_sizeof<Host>::value>::impl_.deallocate(p);
		}

		///	㳰ꤲʤnew黻
		static void* operator new(std::size_t size, const std::nothrow_t& nt) throw()
		{
			return operator new(size, nt, forever);
		}
		///	㳰ꤲʤnew黻Ҥбdelete黻
		static void operator delete(void* p, const std::nothrow_t&) throw()
		{
			operator delete(p);
		}

		///	㳰ꤲʤƱˡդnew黻
		template <typename Sync>
		static void* operator new(std::size_t, const std::nothrow_t&, Sync sync) throw()
		{
			void* p = 0;
			try
			{
				p = allocator<rounded_sizeof<Host>::value>::impl_.allocate(sync);
			}
			catch (...)
			{
			}
			return p;
		}
		///	㳰ꤲʤƱˡդnew黻Ҥбdelete黻
		template <typename Sync>
		static void operator delete(void* p, const std::nothrow_t&, Sync) throw()
		{
			allocator<rounded_sizeof<Host>::value>::impl_.deallocate(p);
		}

		///	new黻Ҥ˼Ԥݤνؿ
		handler_type set_new_handler(handler_type handler)
		{
			return allocator<rounded_sizeof<Host>::value>::impl_.set_new_handler(handler);
		}

	private:
		template <std::size_t Size>
		struct allocator
		{
			static Allocator<Size, Pool> impl_;
		};
	};

	template <class Host, class Pool, template <std::size_t, class> class Allocator = fixed_sized_allocator>
		template <std::size_t Size>
	Allocator<Size, Pool> fixed_sized_new<Host, Pool, Allocator>::allocator<Size>::impl_;


	/**
	 *	@class	variable_sized_new allocator.hpp <toppers/allocator.hpp>
	 *	@brief	סѤ new  delete 黻
	 *	@ingroup	Allocator
	 *
	 *	 @e variable_sized_new ϡƥץ졼Ȱ@e Pool ǻꤷĹסѤ new  delete 
	 *	黻ҤƤ롣\n
	 *	 @e variable_sized_new  public Ѿ뤳ȤˤꡢΥ饹 new  delete 黻Ҥ򥰥Х
	 *	new  delete 黻Ҥ֤뤳ȤǤ롣\n
	 *	 ƥץ졼Ȱ@e Host ˤϡ@e variable_sized_new 륯饹ΤΤꤹ뤳ȡ
	 */
	template <class Host, class Pool>
	class variable_sized_new
	{
		static Pool pool_;
		static void (*handler_)();

	protected:
		///	ǥȥ饯
		/**
		 *	variable_sized_newΥݥ󥿤Ѥ饹delete뤳Ȥʤ褦
		 *	protected°Ȥ롣
		 */
		~variable_sized_new() {}

	public:
		///	new 黻ҥ顼ؿ
		typedef void (*handler_type)();

		///	new黻
		static void* operator new(std::size_t size) throw(std::bad_alloc)
		{
			void* p = 0;
			while (pool_.get(size, &p) != E_OK)
			{
				if (handler_ == 0)
					throw_bad_alloc();
				(*handler_)();
			}
			return p;
		}
		///	delete黻
		static void operator delete(void* p) throw()
		{
			pool_.release(p);
		}

		/**
		 *	@brief	Ʊˡդ new 黻
		 *	@param	size	դ֥åΥʥХȿ
		 *	@param	sync	ƱˡforeverpollingӥॢȻ֤Ǥ롣
		 */
		template <typename Sync>
		static void* operator new(std::size_t size, Sync sync) throw(std::bad_alloc)
		{
			void* p = 0;
			while (pool_.get(size, &p, sync) != E_OK)
			{
				if (handler_ == 0)
					throw_bad_alloc();
				(*handler_)();
			}
			return p;
		}
		///	Ʊˡդnew黻Ҥбdelete黻
		template <typename Sync>
		static void operator delete(void* p, Sync) throw()
		{
			pool_.release(p);
		}

		///	㳰ꤲʤnew黻
		static void* operator new(std::size_t size, const std::nothrow_t& nt) throw()
		{
			void* p = 0;
			try
			{
				p = operator new(size);
			}
			catch (...)
			{
			}
			return p;
		}
		///	㳰ꤲʤnew黻Ҥбdelete黻
		static void operator delete(void* p, const std::nothrow_t&) throw()
		{
			operator delete(p);
		}

		///	㳰ꤲʤƱˡդnew黻
		template <typename Sync>
		static void* operator new(std::size_t size, const std::nothrow_t&, Sync sync) throw()
		{
			void* p = 0;
			try
			{
				p = operator new(size, sync);
			}
			catch (...)
			{
			}
			return p;
		}
		///	㳰ꤲʤƱˡդnew黻Ҥбdelete黻
		template <typename Sync>
		static void operator delete(void* p, const std::nothrow_t&, Sync sync) throw()
		{
			operator delete(p, sync);
		}

		///	new黻ҡѡ
		static void* operator new[](std::size_t size) throw(std::bad_alloc)
		{
			return operator new(size);
		}
		///	delete黻ҡѡ
		static void operator delete[](void* p) throw()
		{
			operator delete(p);
		}

		///	Ʊˡդnew黻ҡѡ
		template <typename Sync>
		static void* operator new[](std::size_t size, Sync sync) throw(std::bad_alloc)
		{
			return operator new(size, sync);
		}
		///	Ʊˡդnew黻Ҥбdelete黻ҡѡ
		template <typename Sync>
		static void operator delete[](void* p, Sync) throw()
		{
			operator delete(p);
		}

		///	㳰ꤲʤnew黻ҡѡ
		static void* operator new[](std::size_t size, const std::nothrow_t& nt) throw()
		{
			return operator new(size, nt);
		}
		///	㳰ꤲʤnew黻Ҥбdelete黻ҡѡ
		static void operator delete[](void* p, const std::nothrow_t& nt) throw()
		{
			operator delete(p, nt);
		}

		///	㳰ꤲʤƱˡդnew黻ҡѡ
		template <typename Sync>
		static void* operator new[](std::size_t size, const std::nothrow_t& nt, Sync sync) throw()
		{
			return operator new(size, nt, sync);
		}
		///	㳰ꤲʤƱˡդnew黻Ҥбdelete黻ҡѡ
		template <typename Sync>
		static void operator delete[](void* p, const std::nothrow_t& nt, Sync sync) throw()
		{
			operator delete(p, nt, sync);
		}

		///	new黻Ҥ˼Ԥݤνؿ
		handler_type set_new_handler(handler_type handler)
		{
			handler_type prev = handler_;
			handler_ = handler;
			return prev;
		}
	};

	template <class Host, class Pool>
	Pool variable_sized_new<Host, Pool>::pool_;

	template <class Host, class Pool>
	void (*variable_sized_new<Host, Pool>::handler_)() = 0;


	/**
	 *	@class	fixed_simple_pool	allocator.hpp	<toppers/allocator.hpp>
	 *	@brief	¾ΤʤĹס
	 *	@ingroup	Allocator
	 *
	 *	 @e fixed_simple_pool ƥץ졼ȥ饹ϡƥץ졼Ȱ@e Size ǻꤷ
	 *	֥å@e N ĳդñʸĹסǤ롣\n
	 *	 @e fixed_simple_pool ϡ¤ñ㲽뤳Ȥǹ®ޤäƤ뤿ᡢ¾ϹԤäƤʤ
	 */
	template <std::size_t N = 32, std::size_t Size = 32>
	class fixed_simple_pool
	{
		union node
		{
			node* next;
			char data[Size];
		};
		union pool
		{
			node buf[N];
			detail::max_alignment_helper force_aligner;
		};
		node* top_;
		pool pool_;

	public:
		///	󥹥ȥ饯
		fixed_simple_pool() : top_(pool_.buf)
		{
			node* next = 0;
			for (node* cur = &pool_.buf[N - 1]; cur != top_; cur--)
			{
				cur->next = next;
				next = cur;
			}
		}

		///	Ĺ֥åγ
		ER get(VP* p_blk)
		{
			node* p = top_;
			top_ = p->next;
			*p_blk = p;
			return E_OK;
		}

		///	Ĺ֥åγƱˡȤθߴѡ
		ER get(VP* p_blk, int sync)
		{
			return get(p_blk);
		}

		///	Ĺ֥åγʥȤθߴѡ
		ER get(UINT size, VP* p_blk)
		{
			if (size > Size)
				return E_PAR;
			node* p = top_;
			top_ = p->next;
			*p_blk = p;
			return E_OK;
		}

		///	Ĺ֥åγʥꤪƱˡȤθߴѡ
		ER get(UINT size, VP* p_blk, int sync)
		{
			return get(size, p_blk);
		}

		///	Ĺ֥åβ
		ER release(VP blk)
		{
			node* p = static_cast<node*>(blk);
			p->next = top_;
			top_ = p;
		}
	};


	/**
	 *	@class	variable_simple_pool	allocator.hpp	<toppers/allocator.hpp>
	 *	@brief	¾ΤʤĹס
	 *	@ingroup	Allocator
	 *
	 *	 @e variable_simple_pool ƥץ졼ȥ饹ϡƥץ졼Ȱ@e TotalSize ǻꤷ
	 *	Ƭʬ䤹ñʲĹסǤ롣\n
	 *	 @e variable_simple_pool ϡ¤ñ㲽뤳Ȥǹ®ޤäƤ뤿ᡢ¾ϹԤäƤʤ
	 *
	 *	@attention	Υסˤäưٳդ֥åϡס뤬ޤǺѤǤʤ
	 */
	template <std::size_t TotalSize = 256>
	class variable_simple_pool
	{
		union pool
		{
			char buf[TotalSize];
			detail::max_alignment_helper force_aligner;
		};
		char* top_;
		pool pool_;

	public:
		///	󥹥ȥ饯
		variable_simple_pool() : top_(pool_.buf)
		{
		}

		///	Ĺ֥åγ
		ER get(UINT size, VP* p_blk)
		{
			typedef	detail::max_alignment_helper	aligner_type;
			size = (size + sizeof(aligner_type) - 1) / sizeof(aligner_type) * sizeof(aligner_type);
			if (p_blk == 0)
				return E_PAR;
			if (top_ + size >= pool_.buf + TotalSize)
				return E_NOMEM;
			*p_blk = top_;
			top_ += size;
			return E_OK;
		}

		///	Ĺ֥åγƱˡȤθߴѡ
		ER get(UINT size, VP* p_blk, int sync)
		{
			return get(size, p_blk);
		}

		///	Ĺ֥åβ
		ER release(VP blk)
		{
			return E_OK;
		}
	};

	/**
	 *	@class	fixed_new_pool	allocator.hpp	<toppers/allocator.hpp>
	 *	@brief	Х new  delete 黻ҤˤĹס
	 *	@ingroup	Allocator
	 *
	 *	 @e fixed_new_pool ƥץ졼ȥ饹ϡХ new  delete 黻ҤѤơ
	 *	Ĺסκۤ饹Ǥ롣
	 *	סѤ٤ȤˤΥ饹ꤹ뤳ȤǡХ new
	 *	 delete 黻ҤѤưѹ뤳ȤǤ롣
	 */
	template <std::size_t Size = 32>
	class fixed_new_pool
	{
	public:
		///	Ĺ֥åγ
		ER get(VP* p_blk)
		{
			if (p_blk == 0)
				return E_PAR;
			void* p = ::operator new(Size, std::nothrow);
			if (p == 0)
				return E_NOMEM;
			*p_blk = p;
			return E_OK;
		}

		///	Ĺ֥åγƱˡȤθߴѡ
		ER get(VP* p_blk, int sync)
		{
			return get(p_blk);
		}

		///	Ĺ֥åγʥȤθߴѡ
		ER get(UINT size, VP* p_blk)
		{
			if (size > Size)
				return E_PAR;
			return get(p_blk);
		}

		///	Ĺ֥åγʥꤪƱˡȤθߴѡ
		ER get(UINT size, VP* p_blk, int sync)
		{
			return get(size, p_blk);
		}

		///	Ĺ֥åβ
		ER release(VP blk)
		{
			::operator delete(blk);
			return E_OK;
		}
	};

	/**
	 *	@class	variable_new_pool	allocator.hpp	<toppers/allocator.hpp>
	 *	@brief	Х new  delete 黻ҤˤĹס
	 *	@ingroup	Allocator
	 *
	 *	 @e variable_new_pool 饹ϡХ new  delete 黻ҤѤơ
	 *	Ĺסκۤ饹Ǥ롣
	 *	סѤ٤ȤˤΥ饹ꤹ뤳ȤǡХ new
	 *	 delete 黻ҤѤưѹ뤳ȤǤ롣
	 */
	class variable_new_pool
	{
	public:
		///	Ĺ֥åγʥȤθߴѡ
		ER get(UINT size, VP* p_blk)
		{
			if (p_blk == 0)
				return E_PAR;
			void* p = ::operator new(size);
			if (p == 0)
				return E_NOMEM;
			*p_blk = p;
			return E_OK;
		}

		///	Ĺ֥åγʥꤪƱˡȤθߴѡ
		ER get(UINT size, VP* p_blk, int sync)
		{
			return get(size, p_blk);
		}

		///	Ĺ֥åβ
		ER release(VP blk)
		{
			::operator delete(blk);
			return E_OK;
		}
	};

}

#endif	// ! TOPPERS_ALLOCATOR_HPP_
