/*
 *  TOPPERS/C++ Binding
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      C++ Binding
 *
 *  Copyright (C) 2004 by TAKAGI Nobuhisa
 *
 *  嵭Ԥϡʲ (1)(4) ξ狼Free Software Foundation
 *  ˤäƸɽƤ GNU General Public License  Version 2 ˵
 *  ҤƤ˸¤ꡤܥեȥܥեȥ
 *  ѤΤޤࡥʲƱˤѡʣѡۡʰʲ
 *  ѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơŬѲǽ
 *  ޤơʤݾڤԤʤޤܥեȥѤˤľ
 *  ŪޤϴŪʤ»˴ؤƤ⡤Ǥʤ
 *
 *  @(#) $Id: object.hpp,v 1.3 2004/01/26 00:53:45 takagi-n Exp $
 */

/**
 *	@file	toppers/object.hpp
 *	@brief	ƥͥ륪֥ȤǶ̤Ѥ륯饹
 *
 *	 ΥإåեǤϰʲΥƥץ졼ȥ饹Ƥ롣
 *
 *	@code
 *	class object_id<Id>;
 *	class object_controller<Id, Object, Stub>;
 *	@endcode
 */

#ifndef	TOPPERS_OBJECT_HPP_
#define	TOPPERS_OBJECT_HPP_

#include <toppers/raw_stub.hpp>
#include <toppers/assert.hpp>
#include <toppers/diagnostics.hpp>

#ifndef	TOPPERS_DEFAULT_STUB
#define	TOPPERS_DEFAULT_STUB	raw_stub<>
#endif

namespace toppers
{

	/**
	 *	@class	object_id object.hpp <toppers/object.hpp>
	 *	@brief	ͥ륪֥ȤIDֹ륯饹
	 *	@ingroup	KernelObjectManager
	 *
	 *	 @e object_id ƥץ졼ȥ饹ϥͥ륪֥ȤIDֹ򰷤Υ饹Ǥ롣
	 *	ƥץ졼Ȱ@e Id ϡ̾ե졼 kernel_id.h 
	 *	IDֹꤹ뤬 0 ꤹ뤳ȤIDֹѿȤݻ뤳ȤǽǤ롣
	 *	ξˤϡ¹Ի˴ϢդƤIDֹꡦѹ뤳ȤǤ롣
	 *
	 *	@attention	IDֹ椬ξˤбƤʤ
	 */
	template <ID Id>
	class object_id
	{
		TOPPERS_STATIC_ASSERT(Id > 0);

	public:
		///	󥹥ȥ饯
		object_id() {}
		///	IDֹΥ󥹥ȥ饯
		explicit object_id(ID id)
		{
			TOPPERS_ASSERT(id == Id);
		}
		///	ͥ륪֥IDֹϢդ
		void attach(ID) {}
		///	ͥ륪֥IDֹڤΥ
		ID detach() { return Id; }
		///	ϢդƤ륫ͥ륪֥IDֹλ
		ID id() const { return Id; }
	};

	template <>
	class object_id<0>
	{
		ID id_;

	public:
		explicit object_id(ID id = 0) : id_(id) {}
		void attach(ID id) { id_ = id; }
		ID detach() { ID prev = id_; id_ = 0; return prev; }
		ID id() const { return id_; }
	};

	namespace detail
	{

		template <object_type Object> struct object_control_helper;

		template <>
		struct object_control_helper<obj_semaphore>
		{
			typedef	t_csem	creation;
			typedef	t_rsem	reference;

			template <class Stub>
			struct syscall
			{
				static ER create(ID objid, creation* pk_cobj)
				{
					return Stub::stub_cre_sem(objid, pk_cobj);
				}
				static ER create(ID objid, const creation* pk_cobj)
				{
					return Stub::stub_cre_sem(objid, pk_cobj);
				}
				static ER_ID create(creation* pk_cobj)
				{
					return Stub::stub_acre_sem(pk_cobj);
				}
				static ER_ID create(const creation* pk_cobj)
				{
					return Stub::stub_acre_sem(pk_cobj);
				}
				static ER destroy(ID objid)
				{
					return Stub::stub_del_sem(objid);
				}
				static ER refer(ID objid, reference* pk_robj)
				{
					return Stub::stub_ref_sem(objid, pk_robj);
				}
			};
		};

		template <>
		struct object_control_helper<obj_eventflag>
		{
			typedef	t_cflg	creation;
			typedef	t_rflg	reference;

			template <class Stub>
			struct syscall
			{
				static ER create(ID objid, creation* pk_cobj)
				{
					return Stub::stub_cre_flg(objid, pk_cobj);
				}
				static ER create(ID objid, const creation* pk_cobj)
				{
					return Stub::stub_cre_flg(objid, pk_cobj);
				}
				static ER_ID create(creation* pk_cobj)
				{
					return Stub::stub_acre_flg(pk_cobj);
				}
				static ER_ID create(const creation* pk_cobj)
				{
					return Stub::stub_acre_flg(pk_cobj);
				}
				static ER destroy(ID objid)
				{
					return Stub::stub_del_flg(objid);
				}
				static ER refer(ID objid, reference* pk_robj)
				{
					return Stub::stub_ref_flg(objid, pk_robj);
				}
			};
		};

		template <>
		struct object_control_helper<obj_dataqueue>
		{
			typedef	t_cdtq	creation;
			typedef	t_rdtq	reference;

			template <class Stub>
			struct syscall
			{
				static ER create(ID objid, creation* pk_cobj)
				{
					return Stub::stub_cre_dtq(objid, pk_cobj);
				}
				static ER create(ID objid, const creation* pk_cobj)
				{
					return Stub::stub_cre_dtq(objid, pk_cobj);
				}
				static ER_ID create(creation* pk_cobj)
				{
					return Stub::stub_acre_dtq(pk_cobj);
				}
				static ER_ID create(const creation* pk_cobj)
				{
					return Stub::stub_acre_dtq(pk_cobj);
				}
				static ER destroy(ID objid)
				{
					return Stub::stub_del_dtq(objid);
				}
				static ER refer(ID objid, reference* pk_robj)
				{
					return Stub::stub_ref_dtq(objid, pk_robj);
				}
			};
		};

		template <>
		struct object_control_helper<obj_mailbox>
		{
			typedef	t_cmbx	creation;
			typedef	t_rmbx	reference;

			template <class Stub>
			struct syscall
			{
				static ER create(ID objid, creation* pk_cobj)
				{
					return Stub::stub_cre_mbx(objid, pk_cobj);
				}
				static ER create(ID objid, const creation* pk_cobj)
				{
					return Stub::stub_cre_mbx(objid, pk_cobj);
				}
				static ER_ID create(creation* pk_cobj)
				{
					return Stub::stub_acre_mbx(pk_cobj);
				}
				static ER_ID create(const creation* pk_cobj)
				{
					return Stub::stub_acre_mbx(pk_cobj);
				}
				static ER destroy(ID objid)
				{
					return Stub::stub_del_mbx(objid);
				}
				static ER refer(ID objid, reference* pk_robj)
				{
					return Stub::stub_ref_mbx(objid, pk_robj);
				}
			};
		};

		template <>
		struct object_control_helper<obj_mutex>
		{
			typedef	t_cmtx	creation;
			typedef	t_rmtx	reference;

			template <class Stub>
			struct syscall
			{
				static ER create(ID objid, creation* pk_cobj)
				{
					return Stub::stub_cre_mtx(objid, pk_cobj);
				}
				static ER create(ID objid, const creation* pk_cobj)
				{
					return Stub::stub_cre_mtx(objid, pk_cobj);
				}
				static ER_ID create(creation* pk_cobj)
				{
					return Stub::stub_acre_mtx(pk_cobj);
				}
				static ER_ID create(const creation* pk_cobj)
				{
					return Stub::stub_acre_mtx(pk_cobj);
				}
				static ER destroy(ID objid)
				{
					return Stub::stub_del_mtx(objid);
				}
				static ER refer(ID objid, reference* pk_robj)
				{
					return Stub::stub_ref_mtx(objid, pk_robj);
				}
			};
		};

		template <>
		struct object_control_helper<obj_messagebuffer>
		{
			typedef	t_cmbf	creation;
			typedef	t_rmbf	reference;

			template <class Stub>
			struct syscall
			{
				static ER create(ID objid, creation* pk_cobj)
				{
					return Stub::stub_cre_mbf(objid, pk_cobj);
				}
				static ER create(ID objid, const creation* pk_cobj)
				{
					return Stub::stub_cre_mbf(objid, pk_cobj);
				}
				static ER_ID create(creation* pk_cobj)
				{
					return Stub::stub_acre_mbf(pk_cobj);
				}
				static ER_ID create(const creation* pk_cobj)
				{
					return Stub::stub_acre_mbf(pk_cobj);
				}
				static ER destroy(ID objid)
				{
					return Stub::stub_del_mbf(objid);
				}
				static ER refer(ID objid, reference* pk_robj)
				{
					return Stub::stub_ref_mbf(objid, pk_robj);
				}
			};
		};

		template <>
		struct object_control_helper<obj_rendezvousport>
		{
			typedef	t_cpor	creation;
			typedef	t_rpor	reference;

			template <class Stub>
			struct syscall
			{
				static ER create(ID objid, creation* pk_cobj)
				{
					return Stub::stub_cre_por(objid, pk_cobj);
				}
				static ER create(ID objid, const creation* pk_cobj)
				{
					return Stub::stub_cre_por(objid, pk_cobj);
				}
				static ER_ID create(creation* pk_cobj)
				{
					return Stub::stub_acre_por(pk_cobj);
				}
				static ER_ID create(const creation* pk_cobj)
				{
					return Stub::stub_acre_por(pk_cobj);
				}
				static ER destroy(ID objid)
				{
					return Stub::stub_del_por(objid);
				}
				static ER refer(ID objid, reference* pk_robj)
				{
					return Stub::stub_ref_por(objid, pk_robj);
				}
			};
		};

		template <>
		struct object_control_helper<obj_fmempool>
		{
			typedef	t_cmpf	creation;
			typedef	t_rmpf	reference;

			template <class Stub>
			struct syscall
			{
				static ER create(ID objid, creation* pk_cobj)
				{
					return Stub::stub_cre_mpf(objid, pk_cobj);
				}
				static ER create(ID objid, const creation* pk_cobj)
				{
					return Stub::stub_cre_mpf(objid, pk_cobj);
				}
				static ER_ID create(creation* pk_cobj)
				{
					return Stub::stub_acre_mpf(pk_cobj);
				}
				static ER_ID create(const creation* pk_cobj)
				{
					return Stub::stub_acre_mpf(pk_cobj);
				}
				static ER destroy(ID objid)
				{
					return Stub::stub_del_mpf(objid);
				}
				static ER refer(ID objid, reference* pk_robj)
				{
					return Stub::stub_ref_mpf(objid, pk_robj);
				}
			};
		};

		template <>
		struct object_control_helper<obj_vmempool>
		{
			typedef	t_cmpl	creation;
			typedef	t_rmpl	reference;

			template <class Stub>
			struct syscall
			{
				static ER create(ID objid, creation* pk_cobj)
				{
					return Stub::stub_cre_mpl(objid, pk_cobj);
				}
				static ER create(ID objid, const creation* pk_cobj)
				{
					return Stub::stub_cre_mpl(objid, pk_cobj);
				}
				static ER_ID create(creation* pk_cobj)
				{
					return Stub::stub_acre_mpl(pk_cobj);
				}
				static ER_ID create(const creation* pk_cobj)
				{
					return Stub::stub_acre_mpl(pk_cobj);
				}
				static ER destroy(ID objid)
				{
					return Stub::stub_del_mpl(objid);
				}
				static ER refer(ID objid, reference* pk_robj)
				{
					return Stub::stub_ref_mpl(objid, pk_robj);
				}
			};
		};

		template <>
		struct object_control_helper<obj_task>
		{
			typedef	t_ctsk	creation;
			typedef	t_rtsk	reference;

			template <class Stub>
			struct syscall
			{
				static ER create(ID objid, creation* pk_cobj)
				{
					return Stub::stub_cre_tsk(objid, pk_cobj);
				}
				static ER create(ID objid, const creation* pk_cobj)
				{
					return Stub::stub_cre_tsk(objid, pk_cobj);
				}
				static ER_ID create(creation* pk_cobj)
				{
					return Stub::stub_acre_tsk(pk_cobj);
				}
				static ER_ID create(const creation* pk_cobj)
				{
					return Stub::stub_acre_tsk(pk_cobj);
				}
				static ER destroy(ID objid)
				{
					return Stub::stub_del_tsk(objid);
				}
				static ER refer(ID objid, reference* pk_robj)
				{
					return Stub::stub_ref_tsk(objid, pk_robj);
				}
			};
		};

		template <>
		struct object_control_helper<obj_cyclichandler>
		{
			typedef	t_ccyc	creation;
			typedef	t_rcyc	reference;

			template <class Stub>
			struct syscall
			{
				static ER create(ID objid, creation* pk_cobj)
				{
					return Stub::stub_cre_cyc(objid, pk_cobj);
				}
				static ER create(ID objid, const creation* pk_cobj)
				{
					return Stub::stub_cre_cyc(objid, pk_cobj);
				}
				static ER_ID create(creation* pk_cobj)
				{
					return Stub::stub_acre_cyc(pk_cobj);
				}
				static ER_ID create(const creation* pk_cobj)
				{
					return Stub::stub_acre_cyc(pk_cobj);
				}
				static ER destroy(ID objid)
				{
					return Stub::stub_del_cyc(objid);
				}
				static ER refer(ID objid, reference* pk_robj)
				{
					return Stub::stub_ref_cyc(objid, pk_robj);
				}
			};
		};

		template <>
		struct object_control_helper<obj_alarmhandler>
		{
			typedef	t_calm	creation;
			typedef	t_ralm	reference;

			template <class Stub>
			struct syscall
			{
				static ER create(ID objid, creation* pk_cobj)
				{
					return Stub::stub_cre_alm(objid, pk_cobj);
				}
				static ER create(ID objid, const creation* pk_cobj)
				{
					return Stub::stub_cre_alm(objid, pk_cobj);
				}
				static ER_ID create(creation* pk_cobj)
				{
					return Stub::stub_acre_alm(pk_cobj);
				}
				static ER_ID create(const creation* pk_cobj)
				{
					return Stub::stub_acre_alm(pk_cobj);
				}
				static ER destroy(ID objid)
				{
					return Stub::stub_del_alm(objid);
				}
				static ER refer(ID objid, reference* pk_robj)
				{
					return Stub::stub_ref_alm(objid, pk_robj);
				}
			};
		};

		template <>
		struct object_control_helper<obj_isr>
		{
			typedef	t_cisr	creation;
			typedef	t_risr	reference;

			template <class Stub>
			struct syscall
			{
				static ER create(ID objid, creation* pk_cobj)
				{
					return Stub::stub_cre_isr(objid, pk_cobj);
				}
				static ER create(ID objid, const creation* pk_cobj)
				{
					return Stub::stub_cre_isr(objid, pk_cobj);
				}
				static ER_ID create(creation* pk_cobj)
				{
					return Stub::stub_acre_isr(pk_cobj);
				}
				static ER_ID create(const creation* pk_cobj)
				{
					return Stub::stub_acre_isr(pk_cobj);
				}
				static ER destroy(ID objid)
				{
					return Stub::stub_del_isr(objid);
				}
				static ER refer(ID objid, reference* pk_robj)
				{
					return Stub::stub_ref_isr(objid, pk_robj);
				}
			};
		};

	}

	/**
	 *	@class	object_controller object.hpp <toppers/object.hpp>
	 *	@brief	ͥ륪֥ȤֻȤ椹륯饹
	 *	@ingroup	KernelObjectManager
	 *
	 *	 @e object_controller ƥץ졼ȥ饹ϥͥ륪֥Ȥ
	 *	ӾֻȤ򥫥ץ벽Ƥ롣\n
	 *	 ƥץ졼Ȱ@e Id  0 ʳξˤϥͥ륪֥Ȥˤ
	 *	@e cre_ ϥӥ뤬Ѥ롣ޤ@e Id  0 ξˤ@e acre_
	 *	ϥӥ뤬Ѥ졢줿ͥ륪֥ȤIDֹưŪ
	 *	Ϣդ褦ˤʤäƤ롣
	 */
	template <ID Id, object_type Object, class Stub>
	class object_controller
	{
		typedef	detail::object_control_helper<Object>	helper;
		typedef	typename helper::syscall<Stub>			syscall;
	public:
		typedef	typename helper::creation	creation;
		typedef	typename helper::reference	reference;

		//	֤ͥθߴΤ¿
		static ER create(object_id<Id>& objid, creation* pk_cobj)
		{
			return syscall::create(Id, pk_cobj);
		}
		///	ͥ륪֥Ȥ
		static ER create(object_id<Id>& objid, const creation* pk_cobj)
		{
			return syscall::create(Id, pk_cobj);
		}
		///	ͥ륪֥Ȥκ
		static ER destroy(object_id<Id>& objid)
		{
			return syscall::destroy(Id);
		}
		///	ͥ륪֥Ȥξֻ
		static ER refer(const object_id<Id>& objid, reference* pk_robj)
		{
			return syscall::refer(Id, pk_robj);
		}
	};

	template <object_type Object, class Stub>
	class object_controller<0, Object, Stub>
	{
		typedef	detail::object_control_helper<Object>	helper;
		typedef	typename helper::syscall<Stub>			syscall;
	public:
		typedef	typename helper::creation	creation;
		typedef	typename helper::reference	reference;

		static ER create(object_id<0>& objid, creation* pk_cobj)
		{
			ER_ID erid = syscall::create(pk_cobj);
			if (erid > 0)
			{
				objid.attach(erid);
				erid = 0;
			}
			return erid;
		}
		static ER create(object_id<0>& objid, const creation* pk_cobj)
		{
			ER_ID erid = syscall::create(pk_cobj);
			if (erid > 0)
			{
				objid.attach(erid);
				erid = 0;
			}
			return erid;
		}
		static ER destroy(object_id<0>& objid)
		{
			ER er = syscall::destroy(objid.id());
			if (er == E_OK)
				objid.detach();
			return er;
		}
		static ER refer(const object_id<0>& objid, reference* pk_robj)
		{
			return syscall::refer(objid.id(), pk_robj);
		}
	};

}

#endif	// ! TOPPERS_OBJECT_HPP_
